//******************************************************************************
//*
//*     FULLNAME:  Single-Chip Microcontroller Real-Time Operating System
//*
//*     NICKNAME:  scmRTOS
//*
//*     PROCESSOR: AVR (Atmel)
//*
//*     TOOLKIT:   avr-gcc (GNU)
//*
//*     PURPOSE:   Target Dependent Low-Level Stuff
//*
//*     Version: 3.05
//*
//*     $Revision: 195 $
//*     $Date:: 2008-06-19 #$
//*
//*     Copyright (c) 2003-2008, Harry E. Zhurov
//*
//*     Permission is hereby granted, free of charge, to any person
//*     obtaining  a copy of this software and associated documentation
//*     files (the "Software"), to deal in the Software without restriction,
//*     including without limitation the rights to use, copy, modify, merge,
//*     publish, distribute, sublicense, and/or sell copies of the Software,
//*     and to permit persons to whom the Software is furnished to do so,
//*     subject to the following conditions:
//*
//*     The above copyright notice and this permission notice shall be included
//*     in all copies or substantial portions of the Software.
//*
//*     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//*     EXPRESS  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
//*     MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//*     IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
//*     CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
//*     TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH
//*     THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//*
//*     =================================================================
//*     See http://scmrtos.sourceforge.net for documentation, latest
//*     information, license and contact details.
//*     =================================================================
//*
//******************************************************************************
//*     avr-gcc port by Oleksandr O. Redchuk, Copyright (c) 2007-2008

.nolist

#include <avr/io.h>

#include "scmRTOS_CONFIG.h"
#include "scmRTOS_TARGET_CFG.h"

.list

;-------------------------------------------------------------------------------
;
#if FLASHEND > 0x1FFF
  #define XJMP jmp
  #define XCALL call
#else
  #define XJMP rjmp
  #define XCALL rcall
#endif


;-------------------------------------------------------------------------------
.macro	save_context

        push	r1			; save r1 value
        in	r1,_SFR_IO_ADDR(SREG)	;
        push	r1			; save SREG
        push	r0
        ; r1 saved before SREG
        push	r2
        push	r3
        push	r4
        push	r5
        push	r6
        push	r7
        push	r8
        push	r9
        push	r10
        push	r11
        push	r12
        push	r13
        push	r14
        push	r15
        push	r16
        push	r17
        push	r18
        push	r19
        push	r20
        push	r21
        push	r22
        push	r23
        push	r24
        push	r25
        push	r26
        push	r27
        push	r28
        push	r29
        push	r30
        push	r31
    #ifdef RAMPZ
	in	r31,_SFR_IO_ADDR(RAMPZ)
	push	r31
    #endif
/*
    #ifdef EIND
	in	r31,_SFR_IO_ADDR(EIND)
	push	r31
    #endif
*/
     .endm
;-------------------------------------------------------------------------------
.macro	restore_context
/*
    #ifdef EIND
	pop	r31
	out  _SFR_IO_ADDR(EIND),r31
    #endif
*/
    #ifdef RAMPZ
	pop	r31
	out  _SFR_IO_ADDR(RAMPZ),r31
    #endif
        pop	r31
        pop	r30
        pop	r29
        pop	r28
        pop	r27
        pop	r26
        pop	r25
        pop	r24
        pop	r23
        pop	r22
        pop	r21
        pop	r20
        pop	r19
        pop	r18
        pop	r17
        pop	r16
        pop	r15
        pop	r14
        pop	r13
        pop	r12
        pop	r11
        pop	r10
        pop	r9
        pop	r8
        pop	r7
        pop	r6
        pop	r5
        pop	r4
        pop	r3
        pop	r2
        ; r1 will be restored after SREG
        pop	r0
        pop	r1			; restore SREG
        out	_SFR_IO_ADDR(SREG),r1	;
        pop	r1			; restore r1 value

     .endm


;-------------------------------------------------------------------------------

     .text


;-------------------------------------------------------------------------------
     .global	OS_Start

OS_Start:
	out	_SFR_IO_ADDR(SPH), r25	; set next process Stack Pointer
	out	_SFR_IO_ADDR(SPL), r24
	rjmp L_RestoreContext

;-------------------------------------------------------------------------------
#if scmRTOS_CONTEXT_SWITCH_SCHEME == 0

	.global	OS_ContextSwitcher
OS_ContextSwitcher:
	save_context

    #ifdef __AVR_ENHANCED__
	movw	r30,r24		; Curr_SP_addr
    #else
	mov	r30,r24
	mov	r31,r25
    #endif

	in	r24,_SFR_IO_ADDR(SPL) ; load current stack pointer
	in	r25,_SFR_IO_ADDR(SPH)
	std	Z+0,r24		; save current process's Stack Pointer
	std	Z+1,r25		;
	out	_SFR_IO_ADDR(SPH), r23	; set new process's Stack Pointer
	out	_SFR_IO_ADDR(SPL), r22

L_RestoreContext:
	restore_context
	ret

;-------------------------------------------------------------------------------
#else

	.extern	OS_ContextSwitchHook
	.global CONTEXT_SWITCH_ISR_VECTOR

CONTEXT_SWITCH_ISR_VECTOR:
	save_context

	in	r24,_SFR_IO_ADDR(SPL)	; load current stack pointer
	in	r25,_SFR_IO_ADDR(SPH)	; as argument
	XCALL OS_ContextSwitchHook
	out	_SFR_IO_ADDR(SPH), r25	; set next stack pointer
	out	_SFR_IO_ADDR(SPL), r24	; from return value

L_RestoreContext:
	restore_context
	reti

#endif // scmRTOS_CONTEXT_SWITCH_SCHEME
;-------------------------------------------------------------------------------

      .end
;*******************************************************************************
